package cz.fidentis.analyst.symmetry;

import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshFacetImpl;
import cz.fidentis.analyst.mesh.core.MeshPointImpl;
import javax.vecmath.Vector3d;

/**
 *
 * @author Natália Bebjaková
 * 
 * Representation of the symmetry plane 
 */
public class Plane {
    public double a;
    public double b;
    public double c;
    public double d;
    private static MeshFacet facet;

    /**
     * Creates new plane 
     * 
     * @param a a coordinate 
     * @param b b coordinate
     * @param c c coordinate
     * @param d d coordinate
     */
    public Plane(double a, double b, double c, double d) {
        this.a = a;
        this.b = b;
        this.c = c;
        this.d = d;
    }
    
    /**
     * Normalize the plane
     */
    public void normalize() {
        double normalLength = Math.sqrt(a * a + b * b + c * c);
        
        a /= normalLength;
        b /= normalLength;
        c /= normalLength;
        d /= normalLength;
    }
    
    /**
     * Returns plane that is represented by mesh which can be merged with model  
     * 
     * @param centroid middle point of the plane 
     * @param a a coordinate
     * @param b b coordinate
     * @param scale distance of points given by bounding box
     * @return plane represented as mesh 
     */
    public static SymmetryEstimator createPlaneMesh(Vector3d centroid, Vector3d a, Vector3d b, double scale) {
        Vector3d[] points = new Vector3d[4];
        
        Vector3d aScaled = new Vector3d(a);
        Vector3d bScaled = new Vector3d(b);
        aScaled.scale(scale);
        bScaled.scale(scale);
        
        points[0] = new Vector3d(centroid);
        points[0].sub(aScaled);
        points[0].sub(bScaled);
        
        points[1] = new Vector3d(centroid);
        points[1].sub(aScaled);
        points[1].add(bScaled);
        
        points[2] = new Vector3d(centroid);
        points[2].add(aScaled);
        points[2].add(bScaled);
        
        points[3] = new Vector3d(centroid);
        points[3].add(aScaled);
        points[3].sub(bScaled);
        
        facet = new MeshFacetImpl();
        for (Vector3d point : points) {
            facet.addVertex(new MeshPointImpl(point, null, null));
        }
        facet.calculateVertexNormals();

        SymmetryEstimator planeMesh = new SymmetryEstimator(facet, Config.getDefault());
        
        return planeMesh;
    }
    
    /**
     * Returns string description of the plane
     * 
     * @return description of the plane
     */
    @Override
    public String toString(){
        return "APPROXIMATE PLANE:\n" + a + "\n" + b + "\n" + c + "\n" + d + "\n";
    }
}
